@@ -1,4 +1,4 @@
-# $Id: __init__.py 6111 2009-09-02 21:36:05Z milde $
+# $Id: __init__.py 7317 2012-01-19 11:55:26Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -52,7 +52,7 @@ class Writer(Component):
 
     def __init__(self):
 
-        # Used by HTML and LaTex writer for output fragments:
+        # Used by HTML and LaTeX writer for output fragments:
         self.parts = {}
         """Mapping of document part names to fragments of `self.output`.
         Values are Unicode strings; encoding is up to the client.  The 'whole'
@@ -71,7 +71,8 @@ class Writer(Component):
         """
         self.document = document
         self.language = languages.get_language(
-            document.settings.language_code)
+            document.settings.language_code,
+            document.reporter)
         self.destination = destination
         self.translate()
         output = self.destination.write(self.output)
